% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% handle video mode & splash popups
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

% global vars used by combo box menu


/xmenu.vspace 4 def
/xmenu.hspace 12 def
/xmenu.light white def
/xmenu.dark black def
/xmenu.font font1 def
/xmenu.normal.bg lightgray def
/xmenu.normal.fg black def
/xmenu.selected.fg white def
/xmenu.selected.bg 0x6c6c6c newcolor


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% Create new xmenu.
%
% ( ) ==> ( window )
%
/window.xmenu {
  widget.size array
  dup .type t_xmenu put
} def


% Handle keyboad input.
%
% ( key_in ) ==> ( key_out )
%
/xmenu.input {
  dup 0 eq { return } if

  dup keyEsc eq {
    xmenu 0 xmenu.oldentry put
    window.done
    pop 0
  } if

  dup keyEnter eq {
    window.done
    xmenu xmenu.video eq { video.update } if
    xmenu xmenu.splash eq { splash.update } if
    xmenu xmenu.keytable eq { keytable.update } if
    pop 0
  } if

  dup keyDown eq {
    xmenu 0 get 1 add xmenu.select
    pop 0
  } if 

  dup keyUp eq {
    xmenu 0 get 1 sub xmenu.select
    pop 0
  } if 

  dup keyF1 eq {
    show_help
    pop 0
  } if 

} def


% Calculate menu sizes.
%
% ( ) ==> ( )
%
/xmenu.sizes {
  /xmenu.lheight xmenu.font setfont fontsize exch pop xmenu.vspace dup add add def
  /xmenu.height xmenu 1 get length xmenu.lheight mul def

  /xmenu.width 0 def
  0 1 xmenu 1 get length 1 sub {
    xmenu 1 get exch get strsize pop
    dup xmenu.width gt { /xmenu.width exch def } { pop } ifelse
  } for
  
  /xmenu.width xmenu.width xmenu.hspace dup add add def
} def


% Init and show menu.
%
% ( window ) ==> ( )
%
% xmenu: [ selected_entry [ text0 text1 ... ] x y ]
%
/xmenu.init {
  /xmenu over .xmenu get def

  xmenu.sizes

  /xmenu.x xmenu 2 get def
  /xmenu.y xmenu 3 get def

  xmenu.x 1 sub xmenu.y 1 sub moveto
  xmenu.light xmenu.dark
  xmenu.width 2 add xmenu.height 2 add
  over over savescreen /xmenu.saved exch def
  drawborder

  0 1 xmenu 1 get length 1 sub { xmenu.viewentry } for

  /xmenu.oldentry xmenu 0 get def

  dup .x xmenu.x put
  dup .y xmenu.y put
      .saved xmenu.saved put

} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% Draw xmenu.
%
% ( window ) ==> ( )
%
/xmenu.show {
  window.push
} def


% ( entry ) ==> ( )
/xmenu.viewentry {
  xmenu.font setfont

  dup xmenu.lheight mul xmenu.y add /xmenu.pos.y exch def

  xmenu.x xmenu.pos.y moveto
  dup xmenu 0 get eq { xmenu.selected.bg } { xmenu.normal.bg } ifelse
  setcolor xmenu.width xmenu.lheight fillrect

  dup xmenu 0 get eq { xmenu.selected.fg } { xmenu.normal.fg } ifelse setcolor
  xmenu.x xmenu.hspace add xmenu.pos.y xmenu.vspace add moveto
  xmenu 1 get over get show

  dup xmenu 0 get eq {
    xmenu.x xmenu.pos.y moveto
    xmenu.dark xmenu.light xmenu.width xmenu.lheight drawborder
  } if

  pop

} def


% Select menu entry.
%
% ( new_entry ) ==> ( )
%
/xmenu.select {
  dup 0 lt { xmenu 1 get length add } if
  dup xmenu 1 get length ge { xmenu 1 get length sub } if

  xmenu 0 get over xmenu 0 rot put
  xmenu.viewentry
  xmenu.viewentry
} def



% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
%
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

% video mode selection and function keys


/fkeys.font font1 def
/fkeys.normal black def
/fkeys.high fn_color def

/fkeys.y    461 def
/fkeys.F1.x  10 def
/fkeys.F2.x  93 def
/fkeys.F3.x 223 def
/fkeys.F4.x 385 def
/fkeys.F9.x 486 def

/video.combo.x fkeys.F2.x 15 add def
/video.combo.y fkeys.y 0 sub def

/splash.combo.x syslinux { fkeys.F4.x } { fkeys.F2.x } ifelse 15 add def
/splash.combo.y fkeys.y 0 sub def

/keytable.combo.x fkeys.F9.x 15 add def
/keytable.combo.y fkeys.y 0 sub def


% yast has kernel splashs at least for those sizes
/video.splashsizes [
   640  480
   800  600
  1024  768
  1280 1024
] def


% Create sort key for video modes.
%
% ( vm_index )  ==> ( sort_index )
%
/vmsortindex {
  video.modes.list exch get
  dup
  1 get 16 shl
  exch 2 get add
} def


% swap video mode entries
%
% ( vm_index_1 vm_index_2 )  ==> ( )
%
/vmsortexch {
  over video.modes.list exch get
  over video.modes.list exch get
  video.modes.list
  5 -1 roll rot put
  video.modes.list 3 1 roll put
} def


% Check if we have a splash in that resolution.
%
% ( video_mode_list_entry ) ==> ( true|false )
%
/video.havesplash {
  false exch

  0 2 video.splashsizes length 1 sub {
    over over over
    2 get rot 1 get rot video.splashsizes exch get eq
    rot 1 add video.splashsizes exch get rot eq and
    { exch pop true exch exit } if
  } for

  pop

} def


% Build video mode list.
%
% ( ) ==> ( )
%
/video.modelist.init {
  /xmenu.video 4 array def

  videomodes 1 add dup array /video.modes.text exch def array /video.modes.list exch def
  xmenu.video 1 video.modes.text put

  % add text mode entry

  % [ mode width height supported ]
  video.modes.list 0 [ 0 0 0 1 ] put

  videomodes 0 gt {
    0 1 videomodes 1 sub {
      video.modes.list exch [ over getvideomode ] exch 1 add exch put
    } for
  } if

  % sort video.modes.list

  video.modes.list length 1 gt {
    0 1 video.modes.list length 2 sub {
      dup 1 add 1 video.modes.list length 1 sub {
        over vmsortindex over vmsortindex gt {
          over over vmsortexch
        } if
        pop
      } for
      pop
    } for
  } if

  % create mode strings

  0 1 video.modes.list length 1 sub {
    video.modes.text exch
    dup video.modes.list exch get
    dup 1 get 0 eq {
      pop "Text Mode" put
    } {
      32 malloc dup rot
      dup 2 get exch 1 get
      "%d x %d" 32 5 -1 roll snprintf
      put
    } ifelse
  } for

  % select first mode
  xmenu.video 0 0 put

  % select largest mode the monitor supports

  0 1 video.modes.list length 1 sub {
    video.modes.list over get
    dup 3 get 0 gt exch video.havesplash and {
      xmenu.video 0 rot put
    } {
      pop
    } ifelse
  } for

  /xmenu xmenu.video def xmenu.sizes

  xmenu.video 2 video.combo.x 1 add put
  xmenu.video 3 video.combo.y 1 sub xmenu.height sub put
  /video.combo.width xmenu.width 2 add def
  /video.combo.height xmenu.lheight def

} def


% Update video mode combo box.
%
% ( ) ==> ( )
%
/video.update {
  video.combo.x xmenu.hspace add video.combo.y moveto
  video.combo.width xmenu.hspace sub video.combo.height
  lightgray setcolor fillrect

  xmenu.normal.fg setcolor
  xmenu.font setfont
  video.combo.x xmenu.hspace add
  video.combo.y
  moveto
  xmenu.video 1 get xmenu.video 0 get get show
} def


/splash.default 1 def

% note the space!
/splash.options [
  "splash=0 "
  "splash=silent "
  "splash=verbose "
] def


% Build splash list.
%
% ( ) ==> ( )
%
/splash.list.init {
  /xmenu.splash [ splash.default [ "Native" "Silent" "Verbose" ] 0 0 ] def

  /xmenu xmenu.splash def xmenu.sizes

  xmenu.splash 2 splash.combo.x 1 add put
  xmenu.splash 3 splash.combo.y 1 sub xmenu.height sub put
  /splash.combo.width xmenu.width 2 add def
  /splash.combo.height xmenu.lheight def
} def


% Update splash combo box.
%
% ( ) ==> ( )
%
/splash.update {
  splash.combo.x xmenu.hspace add splash.combo.y moveto
  splash.combo.width xmenu.hspace sub splash.combo.height
  lightgray setcolor fillrect

  xmenu.normal.fg setcolor
  xmenu.font setfont
  splash.combo.x xmenu.hspace add
  splash.combo.y
  moveto
  xmenu.splash 1 get xmenu.splash 0 get get show
} def


% Build keytable list.
%
% ( ) ==> ( )
%
/keytable.list.init {
  /tmp.kt keymaps length array def
  0 1 tmp.kt length 1 sub {
    tmp.kt exch keymaps over get 0 get put
  } for

  /xmenu.keytable [ 0 tmp.kt 0 0 ] def

  /xmenu xmenu.keytable def xmenu.sizes

  xmenu.keytable 2 keytable.combo.x 1 add put
  xmenu.keytable 3 keytable.combo.y 1 sub xmenu.height sub put
  /keytable.combo.width xmenu.width 2 add def
  /keytable.combo.height xmenu.lheight def
} def


% Update keytable.
%
% ( ) ==> ( )
%
/keytable.update {
  /keymap keymaps xmenu.keytable 0 get get 1 get def

  fkeys.F9.x fkeys.y moveto
  fkeys.high setcolor "F9" show

  keytable.combo.x xmenu.hspace add keytable.combo.y moveto
  keytable.combo.width xmenu.hspace sub keytable.combo.height
  lightgray setcolor fillrect

  xmenu.normal.fg setcolor
  xmenu.font setfont
  keytable.combo.x xmenu.hspace add
  keytable.combo.y
  moveto
  xmenu.keytable 1 get xmenu.keytable 0 get get show

} def


% Init function key line.
%
% ( ) ==> ( )
%
/fkeys.init {
  video.modelist.init
  splash.list.init
  keytable.list.init

  fkeys.font setfont

%  fn_color2 setcolor
%  0 fkeys.y 6 sub moveto screen.size pop 1 sub fkeys.y 6 sub lineto
  lightgray setcolor
  0 fkeys.y 5 sub moveto currentpoint
  screen.size fkeys.y 5 sub sub fillrect
  moveto
  white setcolor
  currentpoint exch pop screen.size pop 1 sub exch
  lineto

  fkeys.F1.x fkeys.y moveto
  fkeys.high setcolor "F1" show
  fkeys.normal setcolor "  Sg" show

  syslinux {
    fkeys.F2.x fkeys.y moveto
    fkeys.high setcolor "F2" show

    fkeys.F3.x fkeys.y moveto
    fkeys.high setcolor "F3" show
    fkeys.normal setcolor "  Driver Update" show

    video.update
  } if

  syslinux
  {
    fkeys.F4.x fkeys.y moveto
    fkeys.high setcolor "F4" show

    splash.update
  } {
    fkeys.F2.x fkeys.y moveto
    fkeys.high setcolor "F2" show

    splash.update
  } ifelse

} def


